--                                                                    --
--  package Strings_Edit            Copyright (c)  Dmitry A. Kazakov  --
--  Implementation                                 Luebeck            --
--                                                 Spring, 2000       --
--                                                                    --
--  This  library  is  free software; you can redistribute it and/or  --
--  modify it under the terms of  the  GNU  Library  General  Public  --
--  License  as  published  by  the Free Software Foundation; either  --
--  version 2 of the License, or (at your option) any later version.  --
--                                                                    --
--  This  library is distributed in the hope that it will be useful,  --
--  but WITHOUT ANY WARRANTY; without even the implied  warranty  of  --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  --
--  Library General Public License for more details.                  --
--                                                                    --
--  You  should  have  received  a  copy  of the GNU Library General  --
--  Public  License  along  with  this library; if not, write to the  --
--  Free Software Foundation, Inc.,  675  Mass  Ave,  Cambridge,  MA  --
--  02139, USA.                                                       --
--____________________________________________________________________--

package body Strings_Edit is
   subtype NumberOfDigits is Positive range 1..100;
   Figures : constant String := "0123456789ABCDEF";

   Layout_Error : exception renames IO_Exceptions.Layout_Error;
   Data_Error   : exception renames IO_Exceptions.Data_Error;
   End_Error    : exception renames IO_Exceptions.End_Error;
   --
   -- GetDigit -- Get one digit
   --
   --	 Symbol	- To be decoded
   --
   -- Returns :
   --
   --	 [0..15]  The decoded digit value
   --    [16]     The Symbol is not a digit
   --
   function GetDigit (Symbol : Character) return Natural;
   pragma Inline (GetDigit);
   function GetDigit (Symbol : Character) return Natural is
   begin
      case Symbol is
         when '0' => return 0;
         when '1' => return 1;
         when '2' => return 2;
         when '3' => return 3;
         when '4' => return 4;
         when '5' => return 5;
         when '6' => return 6;
         when '7' => return 7;
         when '8' => return 8;
         when '9' => return 9;
         when 'A' | 'a' => return 10;
         when 'B' | 'b' => return 11;
         when 'C' | 'c' => return 12;
         when 'D' | 'd' => return 13;
         when 'E' | 'e' => return 14;
         when 'F' | 'f' => return 15;
         when others => return 16;
      end case;
   end GetDigit;
   --
   -- Text_Edit
   --
   -- This  is an internal package containing implementation of all text
   -- editing subprograms. 
   --
   package Text_Edit is
      function TrimCharacter
               (  Source : in String;
                  Blank  : in Character := ' '
               )  return String; 
      function TrimSet
               (  Source : in String;
                  Blanks : in Ada.Strings.Maps.Character_Set
               )  return String;
      procedure GetCharacter
                (  Source  : in String;
                   Pointer : in out Integer;
                   Blank   : in Character := ' '
                );
      procedure GetSet
                (  Source  : in String;
                   Pointer : in out Integer;
                   Blanks  : in Ada.Strings.Maps. Character_Set
                );
      procedure PutString
                (  Destination : in out String;
                   Pointer     : in out Integer;
                   Value       : in String;
                   Field       : in Natural := 0;
                   Justify     : in Alignment := Left;
                   Fill        : in Character := ' '
                );
      procedure PutCharacter
                (  Destination : in out String;
                   Pointer     : in out Integer;
                   Value       : in Character;
                   Field       : in Natural := 0;
                   Justify     : in Alignment := Left;
                   Fill        : in Character := ' '
                );
   end Text_Edit;
   package body Text_Edit is separate;

   function Trim
            (  Source : in String;
               Blank  : in Character := ' '
            )  return String renames Text_Edit.TrimCharacter;
   function Trim
            (  Source : in String;
               Blanks : in Ada.Strings.Maps.Character_Set
            )  return String renames Text_Edit.TrimSet;
   procedure Get
             (  Source  : in String;
                Pointer : in out Integer;
                Blank   : in Character := ' '
             )  renames Text_Edit.GetCharacter;
   procedure Get
             (  Source  : in String;
                Pointer : in out Integer;
                Blanks  : in Ada.Strings.Maps. Character_Set
             )  renames Text_Edit.GetSet;
   procedure Put
             (  Destination : in out String;
                Pointer     : in out Integer;
                Value       : in String;
                Field       : in Natural := 0;
                Justify     : in Alignment := Left;
                Fill        : in Character := ' '
             )  renames Text_Edit.PutString;
   procedure Put
             (  Destination : in out String;
                Pointer     : in out Integer;
                Value       : in Character;
                Field       : in Natural := 0;
                Justify     : in Alignment := Left;
                Fill        : in Character := ' '
             )  renames Text_Edit.PutCharacter;

   package Roman_Edit is
      procedure Get
                (  Source  : in String;
                   Pointer : in out Integer;
                   Value   : out Roman;
                   First   : in Roman := Roman'First;
                   Last    : in Roman := Roman'Last;
                   ToFirst : in Boolean := False;
                   ToLast  : in Boolean := False
                );
      function Value
               (  Source  : in String;
                  First   : in Roman := Roman'First;
                  Last    : in Roman := Roman'Last;
                  ToFirst : in Boolean := False;
                  ToLast  : in Boolean := False
               )  return Roman;
      procedure Put
                (  Destination : in out String;
                   Pointer     : in out Integer;
                   Value       : in Roman;
                   LowerCase   : in Boolean := False;
                   Field       : in Natural := 0;
                   Justify     : in Alignment := Left;
                   Fill        : in Character := ' '
                );
      function Image
               (  Value     : in Roman;
                  LowerCase : in Boolean := False
               )  return String;
   end Roman_Edit;
   package body Roman_Edit is separate;

   procedure Get
             (  Source  : in String;
                Pointer : in out Integer;
                Value   : out Roman;
                First   : in Roman := Roman'First;
                Last    : in Roman := Roman'Last;
                ToFirst : in Boolean := False;
                ToLast  : in Boolean := False
             )  renames Roman_Edit.Get;
   function Value
            (  Source  : in String;
               First   : in Roman := Roman'First;
               Last    : in Roman := Roman'Last;
               ToFirst : in Boolean := False;
               ToLast  : in Boolean := False
            )  return Roman renames Roman_Edit.Value;
   procedure Put
             (  Destination : in out String;
                Pointer     : in out Integer;
                Value       : in Roman;
                LowerCase   : in Boolean := False;
                Field       : in Natural := 0;
                Justify     : in Alignment := Left;
                Fill        : in Character := ' '
             )  renames Roman_Edit.Put;

   function Image
            (  Value     : in Roman;
               LowerCase : in Boolean := False
            )  return String renames Roman_Edit.Image;

   package body Integer_Edit is separate;
   package body Float_Edit is separate;

end Strings_Edit;
