--                                                                    --
--  procedure Test                  Copyright (c)  Dmitry A. Kazakov  --
--  self test for String_Edit                      Luebeck            --
--                                                 Spring, 2000       --
--                                                                    --
--  This  library  is  free software; you can redistribute it and/or  --
--  modify it under the terms of  the  GNU  Library  General  Public  --
--  License  as  published  by  the Free Software Foundation; either  --
--  version 2 of the License, or (at your option) any later version.  --
--                                                                    --
--  This  library is distributed in the hope that it will be useful,  --
--  but WITHOUT ANY WARRANTY; without even the implied  warranty  of  --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  --
--  Library General Public License for more details.                  --
--                                                                    --
--  You  should  have  received  a  copy  of the GNU Library General  --
--  Public  License  along  with  this library; if not, write to the  --
--  Free Software Foundation, Inc.,  675  Mass  Ave,  Cambridge,  MA  --
--  02139, USA.                                                       --
--                                                                    --
with Strings_Edit; use Strings_Edit;
with Text_IO;      use Text_IO;

procedure Test is
   package Float_Edit is new Strings_Edit.Float_Edit (Float);
   package Integer_Edit is new Strings_Edit.Integer_Edit (Integer);
   use Float_Edit;
   use Integer_Edit;
   procedure Put (Text : String; Pointer : Integer) is
      Line  : String (1..80);
      Index : Integer := Line'First;
   begin
      Put (Text);
      New_Line;
      Put
      (  Line,
         Index,
         "",
         Field => Pointer - Text'First,
	 Justify => Right
      );
      Put (Line, Index, "^ Pointer ");
      Integer_Edit.Put (Line, Index, Pointer);
      Put (Line (Line'First..Index - 1));
      New_Line;
   end;

   procedure TestJustify
             (  Value   : String;
                Field   : Natural;
                Justify : Alignment;
                Result  : String
             )  is
      Text    : String (1..68);
      Pointer : Integer;
   begin
      Pointer := 1;
      Put (Text, Pointer, Value, Field => Field, Justify => Justify);
      if Text (1..Pointer-1) /= Result then
         raise Data_Error;
      end if;
   end TestJustify;

   procedure TestFloat (Value : Float; Base : NumberBase := 10) is
      Eps     : constant :=
         Float'Compose (1.0, 1 - Float'Machine_Mantissa) * 7.0;
      Text    : String (1..68);
      Temp    : Float;
      Diff    : Float;
      Pointer : Integer;      
   begin
      Pointer := 1;
      Put (Text, Pointer, Value, Base => Base);
      Temp :=
         Float_Edit.Value
         (  Text (Text'First..Pointer - 1),
            Base => Base,
            ToFirst => True,
            ToLast => True
         );
      if Temp /= Value then
         Diff := abs (2.0 * (Temp - Value) / (Temp + Value));
         if Diff > Eps then
            Put
            (  "Base:" & NumberBase'Image (Base)
            &  " Diff:" & Float'Image (Diff)
            &  ">" & Float'Image (Eps)
            );
            New_Line;
            Put (Text (Text'First..Pointer - 1));
            New_Line;
            Text (Pointer..Text'Last) := (Pointer..Text'Last => ' ');
            Put (Float_Edit.Image (Temp, Base => Base));
            New_Line;
	    raise Data_Error;
         end if;
      end if;
   end TestFloat;

   procedure TestRelPrecision
             (  Value  : Float;
                Small  : Integer;
                Result : String
             )  is
      Text    : String (1..68);
      Pointer : Integer;      
   begin
      Pointer := 1;
      Put (Text, Pointer, Value, RelSmall => Small);
      if Text (1..Pointer-1) /= Result then
         raise Data_Error;
      end if;
   end TestRelPrecision;

   procedure TestAbsPrecision
             (  Value  : Float;
                Small  : Integer;
                Result : String
             )  is
      Text    : String (1..68);
      Pointer : Integer;      
   begin
      Pointer := 1;
      Put (Text, Pointer, Value, AbsSmall => Small);
      if Text (1..Pointer-1) /= Result then
         raise Data_Error;
      end if;
   end TestAbsPrecision;

   procedure TestInteger (Value : Integer; Base : NumberBase := 10) is
      Text    : String (1..68);
      Temp    : Integer;
      Pointer : Integer;
   begin
      Pointer := 1;
      Put (Text, Pointer, Value, Base => Base);
      Temp :=
         Integer_Edit.Value
         (  Text (Text'First..Pointer - 1),
            Base => Base
         );
      if (  Temp /= Value
         or else
            (  Integer_Edit.Image (Temp, Base => Base)
            /= Text (Text'First..Pointer - 1)
         )  )
      then
         Put (Text (Text'First..Pointer - 1));
         New_Line;
         Text (Pointer..Text'Last) := (Pointer..Text'Last => ' ');
         Pointer := 1;
         Get (Text, Pointer, Temp, Base => Base);
         Put (Text, Pointer);
	 raise Data_Error;
      end if;
   end TestInteger;
   Text    : String (1..68);
   Pointer : Integer;
begin
   TestJustify ("#", 10, Left,   "#         ");
   TestJustify ("#", 10, Right,  "         #");
   TestJustify ("#", 10, Center, "    #     ");

   TestJustify ("##########", 10, Left,   "##########");
   TestJustify ("##########", 10, Right,  "##########");
   TestJustify ("##########", 10, Center, "##########");

   TestRelPrecision (2.345678, 1, "2");
   TestRelPrecision (2.345678, 2, "2.3");
   TestRelPrecision (2.345678, 3, "2.35");
   TestRelPrecision (2.345678, 4, "2.346");
   TestRelPrecision (2.345678, 5, "2.3457");

   TestAbsPrecision (2.345678, 1, "0");
   TestAbsPrecision (2.345678, 0, "2");
   TestAbsPrecision (2.345678,-1, "2.3");
   TestAbsPrecision (2.345678,-2, "2.35");
   TestAbsPrecision (2.345678,-3, "2.346");
   TestAbsPrecision (2.345678,-4, "2.3457");

   for Base in NumberBase'Range loop
      TestFloat (Float'Last, Base => Base);
      TestFloat (Float'First, Base => Base);
   end loop;

   TestFloat (1.2e37, Base => 2);
   TestFloat (1.2e37, Base => 16);
   TestFloat (1.2e37);
   TestFloat (123.45);
   TestFloat (0.00000012345);
   TestFloat (0.000000000000012345);

   for Number in Roman'Range loop
      Pointer := 1;
      Put (Text, Pointer, Number);
      if (  Strings_Edit.Value (Text (Text'First..Pointer - 1))
         /= Number
         )
      then
         Put
         (  Text (Text'First..Pointer - 1)
         &  "/="
         &  Strings_Edit.Image (Number)
         );
         New_Line;
	 raise Data_Error;
      end if;
   end loop;

   for Base in NumberBase'Range loop
      TestInteger (Integer'Last, Base => Base);
      TestInteger (Integer'First, Base => Base);
   end loop;

   TestInteger (0);
   TestInteger (12345);

   begin
      if 0.0 = Value ("e+15") then null; end if;
      raise Data_Error;
   exception
      when End_Error => null;
   end;
   begin
      if 0 = Integer'(Value ("+")) then null; end if;
      raise Data_Error;
   exception
      when Data_Error => null;
   end;
   begin
      if 0.0 = Value ("+") then null; end if;
      raise Data_Error;
   exception
      when Data_Error => null;
   end;
   begin
      if 0.0 = Value (" + .") then null; end if;
      raise Data_Error;
   exception
      when Data_Error => null;
   end;
   begin
      if 0.0 = Value ("  xx") then null; end if;
      raise Data_Error;
   exception
      when End_Error => null;
   end;
   begin
      Pointer := 0;
      Put (Text, Pointer, "");
      raise Data_Error;
   exception
      when Layout_Error => null;
   end;
   begin
      Pointer := Text'Last;
      Put (Text, Pointer, "AA");
      raise Data_Error;
   exception
      when Layout_Error => null;
   end;
   Pointer := Text'Last;
   Put (Text, Pointer, "A");
   if Pointer /= Text'Last + 1 or Text (Text'Last) /= 'A' then
      raise Data_Error;
   end if;
   Pointer := Text'Last + 1;
   Put (Text, Pointer, "");
   if Pointer /= Text'Last + 1 then
      raise Data_Error;
   end if;
   Put ("Is seems to be OK");
end Test;
